
#!/bin/bash
#
# lvm-gen-udev-rules
#
# Generate rules to automatically activate a logical volume from udev
#

lvname=$1

if [ -z "$lvname" ] ; then
    echo "Usage: lvm-vg-to-udev-rules <lvname>"
    exit 1
fi

vgname=$(/sbin/lvs -o vg_name --noheadings $lvname | uniq 2> /dev/null)
if [ -z "$vgname" ] ; then
    echo "Volume group \"$lvname\" no found"
    exit 2
fi

vgname=$(echo $vgname)
pvnum=$(/sbin/vgs -o pv_count --noheadings $vgname)

while read tmpvg pvuuid pvname; do
    if [ $tmpvg == $vgname ] ; then
	set -- $(/sbin/dmsetup table | grep ${vgname}- )
        while [ $# -gt 0 ]; do
	    case "$1" in
		*:*)
		    major=${1%:*}
		    devtype=$(sed -n "/^Block devices:/{n;: n;s/^[ ]*$major \(.*\)/\1/p;n;b n}" /proc/devices)
		    ;;
	    esac
	    shift
	done
	pvlist="$pvlist $pvuuid"
    fi
done < <(/sbin/pvs -o vg_name,pv_uuid --noheadings)

[ -z "$devtype" ] && devtype=sd
[ "$devtype" = "device-mapper" ] && devtype=dm-

echo "ACTION==\"add\", KERNEL==\"${devtype}*\", ENV{ID_FS_USAGE}==\"raid\", ENV{ID_FS_TYPE}==\"LVM2_member\", ENV{ID_FS_UUID}!=\"\", IMPORT{PROGRAM}=\"collect $vgname $pvlist \$env{ID_FS_UUID}\""
echo "ACTION==\"add\", ENV{COLLECT_$vgname}==\"0\", RUN+=\"/sbin/vgchange -a y $vgname\""
